/*****************************************************************
* ajaxcontent - version 1.1
* Written by Hadi Michael (github.com/hadimichael)
* This script contains the methods for:
* - putting in an ajax request
* - preloading any .png/.jpg/.gif images
* - displaying the contents in a container when they are ready
* The script will also automatically add pages to the history 
* 	stack, this can be used for back buttons.
* All method names and public variable names are preceeded by an
*  "ac_" to help distinguish them.
*****************************************************************/

/********************* put in AJAX request **********************/
var ac_ajaxFlag = false; //used to track if there is a request in progress
var ac_loadSequenceInitiated = false; //used to track load sequence

var ac_lastLocationRequested = '';
var ac_lastContainerRequested = '';

function ajaxcontent(currentLocation, currentContainer, nextLocation, nextContainer) {
	//if an event is currently running, don't let another one start
	if (ac_ajaxFlag) {return;}

	//show the loader
	ac_makeLoaderDiv('visible');

	//raise the flag to indicate that a request is in progress
	ac_ajaxFlag = true;
	
	//indicate that no loadSequence has been initiated
	ac_loadSequenceInitiated = false;
	
	//check that containers exist
	if (!(ac_containerExists(currentContainer) && ac_containerExists(nextContainer))) {
		alert('Container reference is invalid. Please check that the page has loaded and that the containers exist.');
		ac_ajaxFlag = false;
		return;
	}
	
	//add current location and container to history
	if (currentLocation != '' && currentContainer != '') {ac_addToHist(currentLocation,currentContainer);}

	//store information from the this request to be used when refreshing the page
	ac_lastLocationRequested = nextLocation;
	ac_lastContainerRequested = nextContainer;

	//check to see if the page being requested exists
		//set the 'url to check' based on whether we are running test online of offline
		var locationHref = [];
		locationHref = location.href.split("");
		var locationCheck = "";
		for (i=0;i<4;i++) {
			locationCheck = locationCheck + locationHref[i];
		}
		if (locationCheck == "http") {
			//check to see if it exists
			if (!ac_isThere(nextLocation)) {
				//if the page does not exist, then throw an error
				ac_ajaxFlag = false;
				throwError('Page not found. User may be offline.', nextContainer);
				return;
			}
		} else {
			//we are offline
			ac_ajaxFlag = false;
			throwError('Page not found. Content is not being accessed using HTTP.', nextContainer);
			return;
		}

    //create a page_request object - does not work for IE
    var page_request = new XMLHttpRequest();    
 	
    //make an AJAX request 
    page_request.open('GET', nextLocation, true);
    page_request.send(null);
    
    //once ready, start loading the page
    page_request.onreadystatechange = function(){
    	ac_startPageLoad(page_request, nextContainer);
    }
}
/****************************************************************/

/***************** check to see if a page exists ****************/
function ac_isThere(url) {
	var req = new XMLHttpRequest(); // XMLHttpRequest object
	try {
		req.open('HEAD', url, false);
		req.send(null);
		return req.status == 200 ? true : false;
	}
	catch (er) {
		return false;
	}
}
/****************************************************************/

/********************* start loading the page *******************/
function ac_startPageLoad(page_request, nextContainer) {
	clearTimeout(t); //clear timer that may be running
    
    if (page_request.readyState == 4 && (page_request.status==200 || window.location.href.indexOf("http")==-1)) {  	
    	//if the page is ready...
    	if (!ac_loadSequenceInitiated){
    		//if the page is empty, then throw an error
    		if (page_request.responseText == '') {
    			ac_ajaxFlag = false;
    			throwError('There is nothing to retrieve. Connection may have been lost.', nextContainer);
    		} else {
    			//extract images and preload them
		    	var imagesAry = ac_extractIMGs(page_request.responseText);
		    	ac_preloadImages(imagesAry);
		    	//when preloading is complete, display the page
		    	ac_displayPage(page_request.responseText, nextContainer);
    		}
	    	ac_loadSequenceInitiated = true; //stop page from trying to load again
    	}
    } else if (page_request.status==400) {
    	//if "400" is returned, then throw an error
    	ac_ajaxFlag = false;
    	throwError('400: Page does not exist.', nextContainer);
    } else {
    	//if it's not ready, try again in 500ms --this will be trumped by an onreadystatechange
		var t = setTimeout(function (){ac_startPageLoad(page_request,nextContainer);},500);
	}
}
/****************************************************************/

/**************** image extraction and preloading ***************/
function ac_extractIMGs(page) {
	//extract images in a page by looking at 'src' tags in the passed page contents
	var stringSplit = [];
	stringSplit = page.split('');
	
	var imgTag = 'src=\"'; //this is the tag we're looking for
	var imgTypes = ['png','jpg','gif']; //these are the image types we are trying to extract

	var imagesCounter = 0;
	var imagesAry = [];
	imagesAry[imagesCounter] = '';
	
	for (i = 0; i < page.length;i++) {
		var nextFewChars = "";
		for (j = 0; j < imgTag.length;j++) {
			nextFewChars = nextFewChars + stringSplit[i+j];
		}

		if (nextFewChars == imgTag) {
			var k = i + imgTag.length;
			while (stringSplit[k] != '\"') {
				imagesAry[imagesCounter] = imagesAry[imagesCounter] + stringSplit[k];
				k++;				
			}
			
			var getSuffix = [];
			getSuffix = imagesAry[imagesAry.length-1].split('');
			
			for (g = 0; g < imgTypes.length; g++) {
				var imgType = imgTypes[g];
				var suffix = '';
				for (s = 0; s < imgType.length; s++) {
					var trail = imgType.length - s;
					suffix = suffix + getSuffix[getSuffix.length-trail];
				}
				if (suffix == imgType) {imagesCounter++;}
			}
			imagesAry[imagesCounter] = '';
		}
	}
	return imagesAry;
}

//preload images in the imagesAry
var ac_objImgAry = [];
var ac_imgCounter = 0;
function ac_preloadImages(imagesAry) {
	ac_objImgAry = [];
	ac_imgCounter = 0;
	for (var i = 0; i < imagesAry.length; i++)
	{
		ac_objImgAry[i] = new Image();
		ac_objImgAry[i].src = imagesAry[i];
		//increment the counter every time an images loads successfully
		ac_objImgAry[i].onload = function () {ac_imgCounter++;}
	}
}

//check to see when all the images have loaded
function ac_preloadImagesStatus() {
	if (ac_imgCounter == ac_objImgAry.length-1){
		//all loaded...
		return true;
	} else {
		//not yet
		return false;
	}
}
/****************************************************************/
				
/************************ display the page **********************/
var ac_loadCounter = 0;
function ac_displayPage(contents, nextContainer) {
	clearTimeout(t);
	ac_loadCounter++;
	if (ac_loadCounter < 45) {
		ready = ac_preloadImagesStatus(); //check if ready		
		if (ready) {
			//if yes, load in the new content
			document.getElementById(nextContainer).innerHTML = contents;
			
			//indicate request is complete
			ac_ajaxFlag = false;
			
			//hide the loader
			ac_makeLoaderDiv('hidden');
			
			//reset ac_loadCounter and clear timeout
			clearTimeout(t);
			ac_loadCounter = 0;
		} else {
			//if images are not ready keep trying every 200ms for 45 attempts, ie. 9sec
			var t = setTimeout(function () {ac_displayPage(contents,nextContainer);},200);
		}
	} else {
		//if the images never load
		clearTimeout(t);
		ac_loadCounter = 0;
		ac_ajaxFlag = false;
		throwError('Request timed out. Some content took too long to load.', nextContainer);
	}
}
/****************************************************************/

/*********************** history stacking ***********************/
//manage session history
var ac_locationHist = [];
var ac_containerHist = [];

function ac_addToHist(location, containerID) {
	ac_locationHist[ac_locationHist.length] = location; //add a location to history
	ac_containerHist[ac_containerHist.length] = containerID; //add its container to history
}

function ac_clearHistory(toLength) {
	//clear locations to the set length
	while (ac_locationHist.length > toLength) {
		ac_locationHist.length = ac_locationHist.length-1;
	}
	//clear containers to the set length
	while (ac_containerHist.length > toLength) {
		ac_containerHist.length = ac_containerHist.length-1;
	}
}
/****************************************************************/

/********************** control the loader **********************/
function ac_makeLoaderDiv(state){
	//control the loader div if it exists
	if (ac_containerExists('loader')) {
		document.getElementById('loader').style.visibility = state;
	}	
}
/****************************************************************/

/************************ check container ***********************/
function ac_containerExists(containerId) {
	if (containerId == '' || document.getElementById(containerId)) {
		return true;
	} else {
		return false;
	}
}
/****************************************************************/

/**************************** refresh ***************************/
function ajaxcontentRefresh() {
	ajaxcontent('','',ac_lastLocationRequested, ac_lastContainerRequested);
}
/****************************************************************/

/************************ throw an error ************************/
function throwError(errorMessage,container) {
	//set the error
	if (!ac_containerExists(container)) {
		alert('Uh oh! We\'re sorry, but something went wrong...' + '\n' + 'returned message: ' + errorMessage);
	} else {
		document.getElementById(container).innerHTML = '<p>Uh oh! We\'re sorry, but something went wrong... ' + 
				'You could <a href=\"javascript:ajaxcontentRefresh()\"><b>try again</b></a>.</p>' + 
				'<p><i>returned message: ' + errorMessage + '</i></p></td></table>';
	}

	//hide the loader
	ac_makeLoaderDiv('hidden');
}
/****************************************************************/