/*****************************************************************
* ajaxcontent - version 2.0
* Written by Hadi Michael (github.com/hadimichael)
* This script contains the methods for:
* - putting in an ajax request
* - preloading any .png/.jpg/.gif images
* - displaying the contents in a container when they are ready
* The script will also automatically add pages to the history 
* 	stack, this is useful for pages with back buttons.
* All methods and variables are children of 'ajaxcontent'
*****************************************************************/

var ajaxcontent = {
	ajaxFlag: false, //used to track if there already is a request in progress
	
	timeout: 3000, //set timeout duration in ms
	timeoutTimer: null,
	
	//manage session history
	locationHist: [],
	containerHist: [],
	nextLocationRequested: '',
	nextContainerRequested: '',
	
	load:function(currentLocation, currentContainer, nextLocation, nextContainer) {
		//if a request is currently running, don't let another one start ie. do nothing
		if (this.ajaxFlag) {return;}
	 
		//show the loader
		try {document.getElementById('loader').style.visibility = 'visible';} catch(er) {}
	
		//clear interval and raise the flag to indicate that a request is now in progress
		clearInterval(this.timeoutTimer)
		this.ajaxFlag = true;
		
		//add locations and containers to session stack
		if (currentLocation != '' && currentContainer != '') {this.addToHist(currentLocation, currentContainer);}
		this.nextLocationRequested = nextLocation;
		this.nextContainerRequested = nextContainer;
		
	    //create a pageRequest object
		pageRequest = new XMLHttpRequest();  
	   
		//make an AJAX request 
		try {
		    pageRequest.open('GET', nextLocation, true);
		    pageRequest.send(null);
	    } catch(error) {
	    	this.throwError(error);
	    }
	    
	    //once ready, start processing the page
	    pageRequest.onreadystatechange = function(){
		    if (pageRequest.readyState == 4 && pageRequest.status == 200) {
		    	//if request is finished and the page is ready...  	
		    	ajaxcontent.processResponseText(pageRequest.responseText); 
		    	//if the page hasn't loaded after some time, then timeout
		    	setInterval(function(){if(ajaxcontent.ajaxFlag){ajaxcontent.throwError('timed out. Do you have a slow connection?')}}, ajaxcontent.timeout);
		    } else if (pageRequest.readyState == 4  && pageRequest.status == 404) {
		    	ajaxcontent.throwError('page not found');
		    }
		}
	},
	
	/*********************** history stacking ***********************/
	addToHist:function(location, containerID) {
		this.locationHist[this.locationHist.length] = location; //add a location to history
		this.containerHist[this.containerHist.length] = containerID; //add its container to history
	},
	
	clearHistoryTo:function(toLength) {
		//clear locations to the set length
		while (this.locationHist.length > toLength) {
			this.locationHist.length = this.locationHist.length-1;
		}
		//clear containers to the set length
		while (this.containerHist.length > toLength) {
			this.containerHist.length = this.containerHist.length-1;
		}
	},
	/****************************************************************/
	
	/**************** image extraction and preloading ***************/
	processResponseText:function(pagecontents) {
		//extract images in a page by looking at 'src' tags in the past page contents
		var stringSplit = [];
		stringSplit = pagecontents.split('');
		
		var imgTag = 'src=\"'; //this is the tag we're looking for
		var imgTypes = ['png','jpg','gif']; //these are the image types we are trying to extract
	
		var imagesCounter = 0;
		var imagesAry = [];
		imagesAry[imagesCounter] = '';
		
		for (i = 0; i < pagecontents.length;i++) {
			var nextFewChars = "";
			for (j = 0; j < imgTag.length;j++) {
				nextFewChars = nextFewChars + stringSplit[i+j];
			}
	
			if (nextFewChars == imgTag) {
				var k = i + imgTag.length;
				while (stringSplit[k] != '\"') {
					imagesAry[imagesCounter] = imagesAry[imagesCounter] + stringSplit[k];
					k++;				
				}
				
				var getSuffix = [];
				getSuffix = imagesAry[imagesAry.length-1].split('');
				
				for (g = 0; g < imgTypes.length; g++) {
					var imgType = imgTypes[g];
					var suffix = '';
					for (s = 0; s < imgType.length; s++) {
						var trail = imgType.length - s;
						suffix = suffix + getSuffix[getSuffix.length-trail];
					}
					if (suffix == imgType) {imagesCounter++;}
				}
				imagesAry[imagesCounter] = '';
			}
		}
		
		//if there are no images in the page, then just display it
		if (imagesAry.length == 0 || imagesAry[0] == '') {
			this.displayPage(pagecontents);
			return true;
		}
		
		//preload all the images, then display the page when it's done
		var objImgAry = [];
		var imgsLoadedCounter = 0;
		for (var i = 0; i < imagesAry.length; i++)
		{	
			if (imagesAry[i] == '') {imgsLoadedCounter++;continue;} //ignore blanks for now, eventually I need to work out why I'm getting blanks...
			objImgAry[i] = new Image();
			objImgAry[i].src = imagesAry[i];
			objImgAry[i].onerror = function() {ajaxcontent.throwError('images could not be loaded');}
			objImgAry[i].onload = function() {
				imgsLoadedCounter++;
				if (imgsLoadedCounter == imagesAry.length) {
					ajaxcontent.displayPage(pagecontents);
				}
			}
		}
		
		return true;
	},
	/****************************************************************/
	
	/************************* display page *************************/
	displayPage:function(pagecontents) {
		//display the contents
		try {document.getElementById(this.nextContainerRequested).innerHTML = pagecontents;} catch(error) {this.throwError(error);}
		//hide the loader
		try {document.getElementById('loader').style.visibility = 'hidden';} catch(er) {}
		//drop the flag
		this.ajaxFlag = false;
		
		//================= analytics code =================//
		//this is where you tell Google Analytics what page was displayed
		//_gaq.push(['_trackPageview',this.nextLocationRequested]);
		//=============== end analytics code ===============//
	},
	/****************************************************************/
	
	/**************************** refresh ***************************/
	refresh:function() {
		this.load('','',this.nextLocationRequested, this.nextContainerRequested);
	},
	/****************************************************************/
	
	/************************ throw an error ************************/
	throwError:function(errorMessage) {
		ajaxcontent.ajaxFlag = false;
		//set the error
		try {
			document.getElementById(this.nextContainerRequested).innerHTML = '<p>Uh oh! We\'re sorry, but something went wrong... ' + 
					'You could <a href=\"javascript:ajaxcontentRefresh()\"><b>try again</b></a>.</p>' + 
					'<p><i>returned message: ' + errorMessage + '</i></p>';
		} catch(er) {
			alert('Uh oh! We\'re sorry, but something went wrong...' + '\n' + 'returned message: ' + errorMessage);	
		}
	
		//hide the loader
		try {document.getElementById('loader').style.visibility = 'hidden';} catch(er) {}
	},
	/****************************************************************/
}